-- slight adaption of the example given in the wiki ...
  -- who wrote that, BTW? I'd like to give credit where it's due
function newPaddedImage(filename)
  local source=love.image.newImageData(filename)
  local w,h=source:getWidth(),source:getHeight()
  local paddedData={}
  -- Find closest power-of-two.
  local wp=math.pow(2,math.ceil(math.log(w)/math.log(2)))
  local hp=math.pow(2,math.ceil(math.log(h)/math.log(2)))
  -- Only pad if needed:
  if wp~=w or hp~=h then
    local padded=love.image.newImageData(wp,hp)
    padded:paste(source,0,0)
    paddedData.image=love.graphics.newImage(padded)
  else paddedData.image=love.graphics.newImage(source)
  end
  paddedData.w=w
  paddedData.h=h
  return paddedData
end

function loadImages()
  logo1=newPaddedImage("/img/logos/scirathlogo.png")
  logo2=newPaddedImage("/img/logos/scirathpresents.png")
  lovelogo=newPaddedImage("/img/logos/lovelogo.png")
  lualogo=newPaddedImage("/img/logos/lualogo.png")
  background=newPaddedImage("/img/misc/background.png")
  menu=newPaddedImage("/img/misc/menu.png")
  instructions=newPaddedImage("/img/misc/instructions.png")
  highscores=newPaddedImage("/img/misc/highscores.png")
  newhighscore=newPaddedImage("/img/misc/newhighscore.png")
  tile={}
  for i=1,21 do tile[i]=newPaddedImage("/img/tiles/"..i..".png") end
  player=newPaddedImage("/img/ships/player.png")
  playerweapon=newPaddedImage("/img/weapons/player.png")
  enemy=newPaddedImage("/img/ships/enemy.png")
  enemyweapon=newPaddedImage("/img/weapons/enemy.png")
  weaponhitsprite=love.graphics.newImageFont("/img/weapons/hit.png","1234567")
  explosion={}
  for i=1,18 do explosion[i]=newPaddedImage("/img/explosion/"..i..".png") end
  gamefont=love.graphics.newImageFont("/img/misc/font.png"," ABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890%:!?+-=*/,';()\"$#.\\")
  killbar=newPaddedImage("/img/status/killbar.png")
  shieldbar=newPaddedImage("/img/status/shieldbar.png")
  shields={}
  for i=1,10 do shields[i]=newPaddedImage("/img/shields/"..i..".png") end
  shieldend=newPaddedImage("/img/shields/end.png")
  powerup=love.graphics.newImageFont("/img/shields/powerup.png","123456")
end

function loadSounds()
  playerweaponsound=love.audio.newSource("/snd/playerfire.ogg","static")
  enemyweaponsound=love.audio.newSource("/snd/enemyfire.ogg","static")
  weaponhitsound=love.audio.newSource("/snd/hit.ogg","static")
  explodesound=love.audio.newSource("/snd/explode.ogg","static")
  bgmusic=love.audio.newSource("/snd/Bored No. 2.ogg","stream")
end

function loadHighScores()
  love.filesystem.setIdentity("SpaceHorde")
  highscoredata={}
  for i=1,20 do highscoredata[i]=nil end
  if not love.filesystem.exists("highscores.dat") then saveHighScores() return end
  local data=love.filesystem.load("highscores.dat")
  highscoredata=data()
end

function saveHighScores()
  love.filesystem.setIdentity("SpaceHorde")
  local data="return {"
  for i=1,20,2 do
    if highscoredata[i]==nil then data=data.."nil,nil"
    elseif highscoredata[i+1]==nil then data=data.."nil,nil"
    else data=data.."\""..highscoredata[i].."\","..highscoredata[i+1] end
    if i<19 then data=data.."," end
  end
  data=data.."}"
  love.filesystem.write("highscores.dat",data)
end

function love.load()
  -- seed the random() function with the OS time so we don't get the same results every time
  math.randomseed(os.time())
  -- pop the first few numbers; some systems (mine included) seem to have a problem generating them
  for i=1,3 do math.random() end
  loadImages()
  loadSounds()
  initCredits()
  loadHighScores()
  presentstage="logo"
  presentstep=0
  gamestate="present"
  namestring=""
  highscorerung=21
end

function love.draw()
  if gamestate=="present" then drawPresentation() end
  if gamestate=="titlescreen" then drawTitlescreen() end
  if gamestate=="instructions" then drawInstructions() end
  if gamestate=="highscores" then drawHighScores() end
  if gamestate=="playing" then drawGamescreen() end
  if gamestate=="gameover" then drawGameover() end
  if gamestate=="newhighscore" then drawNewHighScore() end
  if gamestate=="credits" then drawCredits() end
end

function love.update(dt)
  if gamestate=="present" then updatePresentation(dt) end
  if gamestate=="newgame" then newGame() end
  if gamestate=="playing" or gamestate=="gameover" or gamestate=="newhighscore" then updateGamescreen(dt) end
  if gamestate=="credits" then updateCredits(dt) end
end

function love.keypressed(k)
  if gamestate=="present" then keypressPresentation(k)
  elseif gamestate=="titlescreen" then keypressTitlescreen(k)
  elseif gamestate=="instructions" or gamestate=="highscores" then
    if gamestate=="instructions" and k~="i" then keypressTitlescreen(k)
    elseif gamestate=="highscores" and k~="h" then keypressTitlescreen(k)
    else gamestate="titlescreen" end
  elseif gamestate=="gameover" then keypressGameover(k)
  elseif gamestate=="newhighscore" then keypressNewHighScore(k)
  elseif gamestate=="credits" then creditspos=0 creditsfade=1 keypressCredits(k)
  end
end

function keypressPresentation(k) gamestate="titlescreen" end

function drawPresentation()
  love.graphics.setBackgroundColor(0,0,16)
  love.graphics.clear()
  local midx,midy=love.graphics.getWidth()/2,love.graphics.getHeight()/2
  if presentstage=="logo" then
    love.graphics.setColor(255,255,255,255*presentstep)
    love.graphics.draw(logo1.image,midx-(logo1.w/2),midy-(logo1.h/2))
  end
  if presentstage=="presents" then
    love.graphics.setColor(255,255,255,255)
    love.graphics.draw(logo1.image,midx-(logo1.w/2),midy-(logo1.h/2))
    love.graphics.setColor(255,255,255,255*presentstep)
    love.graphics.draw(logo2.image,midx-(logo2.w/2),midy-(logo2.h/2))
  end
  if presentstage=="fadeout" then
    love.graphics.setColor(255,255,255,255*presentstep)
    love.graphics.draw(logo2.image,midx-(logo2.w/2),midy-(logo2.h/2))
  end
end

function updatePresentation(dt)
  if presentstage=="logo" then
    if presentstep>=1 then presentstep=0 presentstage="presents" end
    presentstep=presentstep+(dt/2)
  elseif presentstage=="presents" then
    if presentstep>=1 then presentstep=1 presentstage="fadeout" end
    presentstep=presentstep+(dt/2)
  elseif presentstage=="fadeout" then
    if presentstep<=0 then gamestate="titlescreen" end
    presentstep=presentstep-(dt/2)
  end
end

function keypressTitlescreen(k)
  if k=="n" then gamestate="newgame" end
  if k=="i" then gamestate="instructions" end
  if k=="h" then gamestate="highscores" end
  if k=="c" then gamestate="credits" end
  if k=="q" then love.event.push("q") end
end

function drawTitlescreen()
  love.graphics.clear()
  love.graphics.setColor(255,255,255,255)
  love.graphics.draw(background.image,0,0)
  love.graphics.draw(menu.image,0,love.graphics.getHeight()-menu.h)
end

function drawInstructions()
  drawTitlescreen()
  love.graphics.draw(instructions.image,(love.graphics.getWidth()/2)-(instructions.w/2),(love.graphics.getHeight()/2)-(instructions.h/2))
end

function drawHighScores()
  drawTitlescreen()
  local x,y=(love.graphics.getWidth()/2)-(highscores.w/2),(love.graphics.getHeight()/2)-(highscores.h/2)
  love.graphics.draw(highscores.image,x,y)
  love.graphics.setFont(gamefont)
  x=x+94 y=y+54
  for i=1,20,2 do
    local desc="---"
    if highscoredata[i]~=nil then desc=highscoredata[i].." "..tonumber(highscoredata[i+1]) end
    love.graphics.print(desc,x,y)
    y=y+21
  end
end

-- create a random row of stars
function newStarRow()
  local row={}
  for i=1,20 do
    local starcheck=math.random(6)+math.random(6) -- is there a star system here?
    if starcheck<10 then row[i]=1 -- no system present
    else -- start system indicated...
      local systemsize=math.random(6)+math.random(6) -- how big is it?
      if systemsize<=9 then row[i]=math.random(2,5) -- unary
      elseif systemsize==10 then row[i]=math.random(6,15) -- binary
      else row[i]=math.random(16,21) end -- trinary
    end
  end
  return row
end

function initStarfield()
  starfield={}
  for i=1,16 do starfield[i]=newStarRow() end
  rowoffset=0
end

function initPlayer()
  player.x=love.graphics.getWidth()/2-(player.w/2)
  player.y=love.graphics.getHeight()-(player.h*2.5)
  playershots={}
  playerweaponfuse=0
  player.score=0
  player.kills=0
  player.lives=3
  player.shields=10
  player.hitloc={}
  player.isdead=false
  player.isreviving=false
  player.revivefuse=0
end

function initEnemy()
  enemies={}
  maxenemies=3
  enemyshots={}
end

function newGame()
  initStarfield()
  initPlayer()
  initEnemy()
  increasedifficulty=0
  explosionobject={}
  powerups={}
  love.graphics.setColor(255,255,255,255)
  gamestate="playing"
  bgmusic:play()
end

function drawGamescreen()
  -- draw star field
  local y=love.graphics.getHeight()-40+rowoffset
  local x=0
  for i=1,16 do
    for j=1,20 do love.graphics.draw(tile[starfield[i][j]].image,x,y) x=x+40 end
    y=y-40
    x=0
  end
  -- draw player's ship
  if not player.isdead then love.graphics.draw(player.image,math.floor(player.x),math.floor(player.y)) end
  if player.isreviving then
    love.graphics.setColor(255,255,255,64)
    love.graphics.draw(player.image,math.floor(player.x),math.floor(player.y))
    love.graphics.setColor(255,255,255,255)
  end
  -- draw player's shots
  i=1
  while i<=#playershots do
    x,y=math.floor(playershots[i][1]),math.floor(playershots[i][2])
    love.graphics.setColor(255,255,255,128)
    love.graphics.draw(playerweapon.image,x,y)
    love.graphics.draw(playerweapon.image,x+12,y)
    love.graphics.setColor(255,255,255,255)
    i=i+1
  end
  -- draw enemy ships
  i=1
  while i<=#enemies do
    love.graphics.draw(enemy.image,math.floor(enemies[i].x),math.floor(enemies[i].y))
    for j=1,3 do
      if enemies[i].hitloc[j]~=nil then
        love.graphics.setFont(weaponhitsprite)
        local frame=string.format("%d",enemies[i].hitloc[j][3])
        love.graphics.print(frame,enemies[i].hitloc[j][1],enemies[i].hitloc[j][2])
        enemies[i].hitloc[j][3]=enemies[i].hitloc[j][3]+1
        if enemies[i].hitloc[j][3]>7 then enemies[i].hitloc[j]=nil end
        love.graphics.setFont(gamefont)
      end
    end
    i=i+1
  end
  -- draw enemy fire
  i=1
  while i<=#enemyshots do
    x,y=math.floor(enemyshots[i][1]),math.floor(enemyshots[i][2])
    love.graphics.setColor(255,255,255,128)
    love.graphics.draw(enemyweapon.image,x,y)
    love.graphics.setColor(255,255,255,255)
    i=i+1
  end
  -- draw enemy hits
  for i=1,10 do
    if player.hitloc[i]~=nil then
      love.graphics.setFont(weaponhitsprite)
      local frame=string.format("%d",player.hitloc[i][3])
      love.graphics.print(frame,player.hitloc[i][1],player.hitloc[i][2])
      player.hitloc[i][3]=player.hitloc[i][3]+1
      if player.hitloc[i][3]>7 then player.hitloc[i]=nil end
      love.graphics.setFont(gamefont)
    end
  end
  -- draw explosion objects
  i=1
  while i<=#explosionobject do
    love.graphics.draw(explosion[explosionobject[i][3]].image,explosionobject[i][1],explosionobject[i][2])
    explosionobject[i][3]=explosionobject[i][3]+1
    if explosionobject[i][3]>18 then table.remove(explosionobject,i) i=i-1 end
    i=i+1
  end
  -- draw powerups
  i=1
  while i<=#powerups do
    love.graphics.setFont(powerup)
    local frame=string.format("%d",math.ceil(powerups[i].frame))
    love.graphics.print(frame,math.floor(powerups[i].x),math.floor(powerups[i].y))
    love.graphics.setFont(gamefont)
    i=i+1
  end
  -- stats info
  love.graphics.setFont(gamefont)
  local desc=string.format("*%d",player.lives)
  local lifex=love.graphics.getWidth()-10-love.graphics.getFont():getWidth(desc)
  love.graphics.print(desc,lifex,(10+player.h)/2)
  love.graphics.draw(player.image,lifex-10-player.w,10)
  local scorex=lifex+20+player.w
  desc=string.format("%d",player.score)
  love.graphics.print(desc,(love.graphics.getWidth()+love.graphics.getFont():getWidth(desc))/2,10)
  local killx,killy=love.graphics.getWidth()-killbar.w-10,love.graphics.getHeight()-killbar.h-10
  love.graphics.draw(killbar.image,killx,killy)
  desc=string.format("%d",player.kills)
  love.graphics.printf(desc,killx+28,killy+6,42,"center")
  love.graphics.draw(shieldbar.image,10,killy)
  local shieldx,shieldy=33,killy+4
  for i=1,player.shields do
    love.graphics.draw(shields[i].image,shieldx,shieldy)
    shieldx=shieldx+shields[i].w
  end
  love.graphics.draw(shieldend.image,shieldx,shieldy)
  -- 1-up notice
  if oneupcount~=nil and oneupblink==true then
    local midx,midy=love.graphics.getWidth()/2,love.graphics.getHeight()/2
    love.graphics.draw(player.image,midx-(player.w/2),midy-(player.h/2))
    local desc="1-UP!"
    love.graphics.print(desc,midx-(love.graphics.getFont():getWidth(desc)/2),midy-(love.graphics.getFont():getHeight()/2))
  end
end

function addExplosion(x,y) table.insert(explosionobject,{x,y,1}) end

function killPlayer()
  local x,y=player.x+(player.w/2)-(explosion[1].w/2),player.y+(player.h/2)-(explosion[1].h/2)
  addExplosion(x,y)
  explodesound:play()
  player.lives=player.lives-1
  player.isdead=true
  if player.lives>0 then player.isreviving=true player.revivefuse=30
  else
    bgmusic:stop()
    -- check for new high score
    local rung=21
    for i=1,20,2 do
      if rung==21 then
        if highscoredata[i]==nil then rung=i
        elseif player.score>highscoredata[i+1] then rung=i
        elseif player.score==highscoredata[i+1] then rung=i+2 end
      end
    end
    if rung<21 then gamestate="newhighscore" highscorerung=rung
    else gamestate="gameover" end
  end
end

function revivePlayer()
  player.shields=10
  player.isreviving=false
  player.isdead=false
end

function updateGamescreen(dt)
  if bgmusic:isStopped() and gamestate=="playing" then bgmusic:play() end
  -- update starfield
  rowoffset=rowoffset+(100*dt)
  if rowoffset>=40 then rowoffset=0 table.remove(starfield,1) table.insert(starfield,newStarRow()) end
  -- move powerups
  i=1
  while i<=#powerups do
    local y,frame=powerups[i].y,powerups[i].frame
    y=y+(100*dt) frame=frame+(10*dt)
    if frame>6 then frame=1 end
    powerups[i].y=y powerups[i].frame=frame
    if y>love.graphics.getHeight() then table.remove(powerups,i) i=i-1 end
    i=i+1
  end
  -- collect powerups
  if player.isdead==false and player.isreviving==false then
    i=1
    while i<=#powerups do
      local x,y=powerups[i].x,powerups[i].y
      local collected=false
      if y>=player.y-11 and y<=player.y+15 then
        if x>=player.x-14 and x<=player.x+70 then
          if x>=player.x-5 and x<=player.x+51 then collected=true end
          if y>=player.y+6 then collected=true end
        end
        if collected==true then
          table.remove(powerups,i)
          i=i-1
          player.shields=player.shields+3
          if player.shields>10 then player.shields=10 end
        end
      end
      i=i+1
    end
  end
  -- check for player movement
  if love.keyboard.isDown("left") or love.keyboard.isDown("a") then
    player.x=player.x-(250*dt)
    if player.x<0 then player.x=0 end
  end
  if love.keyboard.isDown("right") or love.keyboard.isDown("d") then
    player.x=player.x+(250*dt)
    if player.x+player.w>love.graphics.getWidth() then player.x=love.graphics.getWidth()-player.w end
  end
  if love.keyboard.isDown("up") or love.keyboard.isDown("w") then
    player.y=player.y-(250*dt)
    if player.y<love.graphics.getHeight()*0.75 then player.y=love.graphics.getHeight()*0.75 end
  end
  if love.keyboard.isDown("down") or love.keyboard.isDown("s") then
    player.y=player.y+(250*dt)
    if player.y>love.graphics.getHeight()-player.h then player.y=love.graphics.getHeight()-player.h end
  end
  -- handle player attacks
  i=1
  while i<=#playershots do
    local x,y=playershots[i][1],playershots[i][2]
    y=y-(250*dt)
    if y<(playerweapon.h*-1) then table.remove(playershots,i)
    else playershots[i]={x,y} i=i+1 end
  end
  if playerweaponfuse>0 then
    playerweaponfuse=playerweaponfuse-(19*dt)
    if playerweaponfuse<0 then playerweaponfuse=0 end
  end
  if love.keyboard.isDown(" ") and playerweaponfuse==0 and player.isreviving==false and player.isdead==false then
    playerweaponsound:stop()
    playerweaponsound:play()
    table.insert(playershots,{player.x-9,player.y-46})
    playerweaponfuse=5
  end
  -- check to see if player shots collided with an enemy
  i=1
  while i<=#enemies do
    local target=enemies[i]
    target.h=enemy.h target.w=enemy.w
    j=1
    while j<=#playershots do
      local x,y=playershots[j][1]+29,playershots[j][2]+17
      if y>0 then
        if y>=target.y and y<=target.y+target.h then
          if (x>=target.x and x+5<=target.x+target.w) or (x+12>=target.x and x+17<=target.x+target.w) then
            target.hits=target.hits-1
            local xx=target.x+3
            if x+12>=target.x and x+17<=target.x+target.w then xx=target.x+15 end
            weaponhitsound:play()
            player.score=player.score+((4-target.hits)*5)
            target.hitloc[target.hits]={xx,y,1}
            if target.hits==0 then
              explodesound:play()
              addExplosion((target.x+(target.w/2))-(explosion[1].w/2),(target.y+(target.h/2))-(explosion[1].h/2))
              increasedifficulty=increasedifficulty+1
              if increasedifficulty>=maxenemies then increasedifficulty=0 maxenemies=maxenemies+2 end
              player.kills=player.kills+1
              if player.kills==100 then player.kills=0 player.lives=player.lives+1 oneupcount=1 oneupblink=true end
              table.remove(enemies,i) i=i-1
            end
            table.remove(playershots,j)
            j=j-1
          end
        end
      end
      j=j+1
    end
    i=i+1
    end
  -- end of collision test (player shots)
  -- 1-up notice timer
  if oneupcount~=nil then
    oneupcount=oneupcount+(2*dt)
    if oneupcount>=1 then oneupblink=true end
    if oneupcount>=2 then oneupblink=false end
    if oneupcount>=3 then oneupblink=true end
    if oneupcount>=4 then oneupblink=false end
    if oneupcount>=5 then oneupblink=true end
    if oneupcount>=6 then onupblink=false oneupcount=nil end
  end
  -- move enemies
  i=1
  while i<=#enemies do
    local movecheck=math.random(6)
    if enemies[i].movebias==nil then enemies[i].movebias=movecheck
    else
      movecheck=enemies[i].movebias
      if math.random(20)==1 then enemies[i].movebias=nil end
    end
    local mvamt=(100*dt)/2
    if movecheck==1 then
      enemies[i].x=enemies[i].x-mvamt
      if enemies[i].x<29 then enemies[i].x=29 end
    end
    if movecheck==6 then
      enemies[i].x=enemies[i].x+mvamt
      if enemies[i].x+enemy.w+29>love.graphics.getWidth() then enemies[i].x=love.graphics.getWidth()-enemy.w-29 end
    end
  enemies[i].y=enemies[i].y+mvamt
  if enemies[i].y>love.graphics.getHeight() then table.remove(enemies,i) i=i-1 end
  i=i+1
  end
  -- check to see if enemy fires
  i=1
  while i<=#enemies do
    local firechance=1
    local firezone=enemies[i].x+13
    if enemies[i].shotfuse>0 then
      enemies[i].shotfuse=enemies[i].shotfuse-(15*dt)
      if enemies[i].shotfuse<0 then enemies[i].shotfuse=0 end
    end
    if enemies[i].shotfuse==0 then
      if firezone>=player.x and firezone<=player.x+18 then firechance=40 end
      if firezone>=player.x+19 and firezone<=player.x+37 then firechance=80 end
      if firezone>=player.x+38 and firezone<=player.x+56 then firechance=40 end
      if math.random(400)<= firechance then
        enemyweaponsound:play()
        table.insert(enemyshots,{enemies[i].x-18,enemies[i].y-9})
        enemies[i].shotfuse=5
      end
    end
    i=i+1
  end
  --update enemy fire
  i=1
  while i<=#enemyshots do
    local x,y=enemyshots[i][1],enemyshots[i][2]
    y=y+(250*dt)
    if y>love.graphics.getHeight() then table.remove(enemyshots,i)
    else enemyshots[i]={x,y} i=i+1 end
  end
  --check for enemy weapon collisions
  if player.isdead==false and player.isreviving==false then
    i=1
    while i<=#enemyshots do
      local x,y=enemyshots[i][1]+29,enemyshots[i][2]+43
      local hashit=false
      if y>=player.y and y<=player.y+25 then
        if x>=player.x-4 and x<=player.x+60 then
          if x>=player.x+15 and x<=player.x+41 then hashit=true end
          if y>=player.y+17 then hashit=true end
        end
      end
      if hashit==true then
        weaponhitsound:play()
        player.hitloc[11-player.shields]={x+2,y,1}
        table.remove(enemyshots,i)
        player.shields=player.shields-1
        if player.shields==0 then killPlayer() end
        i=i-1
      end
      i=i+1
    end
  end
  -- end of enemy weapon collisions
  -- see if an enemy has rammed into the player
  if player.isdead==false and player.isreviving==false then
    i=1
    while i<=#enemies do
      local x,y=enemies[i].x,enemies[i].y
      local hasrammed=false
      if y>=player.y-23 and y<=player.y+25 then
        if x>=player.x-26 and x<=player.x+82 then
          if x>=player.x-7 and x<=player.x+63 then hasrammed=true end
          if y>=player.y-6 then hasrammed=true end
        end
        if hasrammed==true then
          addExplosion((enemies[i].x+(enemy.w/2))-(explosion[1].w/2),(enemies[i].y+(enemy.h/2))-(explosion[1].h/2))
          table.remove(enemies,i)
          i=i-1
          killPlayer()
        end
      end
      i=i+1
    end
  end
  -- end of ramming check
  -- add some enemies
  if math.random(100)==1 and #enemies<maxenemies then
    table.insert(enemies,{})
    local x=math.random(29,love.graphics.getWidth()-29)
    enemies[#enemies].x=x enemies[#enemies].y=0-enemy.h
    enemies[#enemies].hits=3 enemies[#enemies].hitloc={nil}
    enemies[#enemies].shotfuse=0
  end
  -- add a powerup
  if math.random(1000)<=10-player.shields and #powerups<math.floor(#enemies/2) then
    table.insert(powerups,{})
    local x=math.random(0,love.graphics.getWidth()-14)
    powerups[#powerups].x=x powerups[#powerups].y=-11
    powerups[#powerups].frame=1
  end
  -- reviving the player...
  if player.isreviving then
    player.revivefuse=player.revivefuse-(15*dt)
    if player.revivefuse<=0 then
      player.revivefuse=0
      revivePlayer()
    end
  end
end

function keypressGameover(k) gamestate="titlescreen" end

function drawGameover()
  drawGamescreen()
  local midx,midy=love.graphics.getWidth()/2,love.graphics.getHeight()/2
  love.graphics.setFont(gamefont)
  local desc="GAME OVER"
  love.graphics.print(desc,midx-(love.graphics.getFont():getWidth(desc)/2),midy-(love.graphics.getFont():getHeight()/2))
end

function keypressNewHighScore(k)
  if string.len(namestring)<10 then
    if k=="a" then namestring=namestring.."A" end
    if k=="b" then namestring=namestring.."B" end
    if k=="c" then namestring=namestring.."C" end
    if k=="d" then namestring=namestring.."D" end
    if k=="e" then namestring=namestring.."E" end
    if k=="f" then namestring=namestring.."F" end
    if k=="g" then namestring=namestring.."G" end
    if k=="h" then namestring=namestring.."H" end
    if k=="i" then namestring=namestring.."I" end
    if k=="j" then namestring=namestring.."J" end
    if k=="k" then namestring=namestring.."K" end
    if k=="l" then namestring=namestring.."L" end
    if k=="m" then namestring=namestring.."M" end
    if k=="n" then namestring=namestring.."N" end
    if k=="o" then namestring=namestring.."O" end
    if k=="p" then namestring=namestring.."P" end
    if k=="q" then namestring=namestring.."Q" end
    if k=="r" then namestring=namestring.."R" end
    if k=="s" then namestring=namestring.."S" end
    if k=="t" then namestring=namestring.."T" end
    if k=="u" then namestring=namestring.."U" end
    if k=="v" then namestring=namestring.."V" end
    if k=="w" then namestring=namestring.."W" end
    if k=="x" then namestring=namestring.."X" end
    if k=="y" then namestring=namestring.."Y" end
    if k=="z" then namestring=namestring.."Z" end
  end
  if k=="backspace" and string.len(namestring)>0 then namestring=string.sub(namestring,1,-2) end
  if k=="return" then
    table.insert(highscoredata,highscorerung,namestring)
    table.insert(highscoredata,highscorerung+1,player.score)
    while #highscoredata>20 do table.remove(highscoredata,#highscoredata) end
    saveHighScores()
    highscorerung=21
    namestring=""
    gamestate="highscores"
  end
end

function drawNewHighScore(k)
  drawGamescreen()
  local x,y=(love.graphics.getWidth()/2)-(newhighscore.w/2),(love.graphics.getHeight()/2)-(newhighscore.h/2)
  love.graphics.draw(newhighscore.image,x,y)
  x=x+41 y=y+71
  love.graphics.printf(namestring,x,y,180,"center")
end

function initCredits()
  creditspos=0
  creditsfade=1
  creditstext=
[[::: CREDITS :::

- MAIN PROGRAMMING -
ALEXANDER D. MATTHEWS, II (A.K.A., "SCIRATH")

- ADDITIONAL LIBRARIES AND CODE -
NEWPADDEDIMAGE() - LOVE DOCUMENTATION
(SLIGHTLY ALTERED)


-=PROGRAMS USED=-
GAME ENGINE - LOVE V.0.7.2 (LOVE2D.ORG)
CODING - NOTEPAD++ V.5.4.6
GRAPHICS EDITING - PAINT SHOP PRO V.9.00
AUDIO CONVERSION - AUDACITY V.1.2.6 AND VLC V.1.1.11


-=SOUND=-
THE PENGUIN DEVELOPMENT TEAM


-=MUSIC=-
"BORED NO. 2" - ALEXANDER D. MATTHEWS, II


-=FONT=-
DANIEL "DANC" COOK


-=GRAPHICS=-

- BACKDROP -
PHILIPPE CHABOT
ALEXANDER D. MATTHEWS, II

- MENU/INFOSCREENS -
DANIEL "DANC" COOK
ALEXANDER D. MATTHEWS, II

- SHIELDBAR/KILLBAR -
DANIEL "DANC" COOK

- SHIPS -
ARI FELDMAN

- LASER EFFECTS -
PHILIPPE CHABOT

- EXPLOSIONS -
DANIEL "DANC" COOK
HANS-MARTIN PORTMAN

- POWER UPS -
ARI FELDMAN

- STAR TILES -
ALEXANDER D. MATTHEWS, II


-=PLAYTESTING=-
ERIC HANCOCK
BILL WODENHELM


]]
  local j=1
  for i=1,string.len(creditstext) do
    if string.sub(creditstext,i,i)=="\n" then j=j+1 end
  end
  love.graphics.setFont(gamefont)
  creditsheight=love.graphics.getFont():getHeight()*j+25+tallestlogo()+love.graphics.getHeight()
end

function tallestlogo()
  if lovelogo.h>lualogo.h then return lovelogo.h end
  return lualogo.h
end

function keypressCredits(k)
  gamestate="titlescreen"
end

function updateCredits(dt)
  if creditspos<creditsheight then creditspos=creditspos+(dt*30) end
  if creditsheight-tallestlogo()-creditspos<(love.graphics.getHeight()/2)-(tallestlogo()/2) then
    creditsfade=creditsfade-(dt/4)
    if creditsfade<0 then creditsfade=0 creditspos=0 gamestate="titlescreen" end
  end
end

function drawCredits()
  love.graphics.clear()
  love.graphics.setColor(255,255,255,255)
  love.graphics.draw(background.image,0,0)
  love.graphics.setColor(255,255,255,255*creditsfade)
  love.graphics.setFont(gamefont)
  love.graphics.printf(creditstext,25,love.graphics.getHeight()+(love.graphics.getFont():getHeight()*2)-creditspos,love.graphics.getWidth()-50,"center")
  local h=tallestlogo()
  local midx,midy=love.graphics.getWidth()/2,love.graphics.getHeight()/2
  if creditsheight-h-creditspos>midy-(h/2) then
    love.graphics.draw(lovelogo.image,midx-(lovelogo.w+35),creditsheight-h-creditspos)
    love.graphics.draw(lualogo.image,midx+15,creditsheight-h-creditspos)
  else
    love.graphics.draw(lovelogo.image,midx-(lovelogo.w+35),midy-(h/2))
    love.graphics.draw(lualogo.image,midx+15,midy-(h/2))
  end
end